import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import java.util.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;

//This class deals with displaying name of entry's.
//Like it will display House 1, House 2, House 3 etc..

public class EntryNamePanel extends JPanel implements ActionListener{

	private static final long serialVersionUID = 1L;
	public static Font LABELFONT = new Font("Arial", Font.PLAIN, 11);
    public static final AffineTransform NINETY = 
	new AffineTransform(0.0, -1.0, 1.0, 0.0, 0.0, 0.0);
    public static final AffineTransform FORTYFIVE = 
	AffineTransform.getRotateInstance(-Math.PI / 5.0);
    public static Font VLABELFONT = LABELFONT.deriveFont(FORTYFIVE);
	
    private static final int ANG_WD = 42;
    private static final int ANG_HT = 30;
	Vector labelList;
	int panelHeight = 0;
	int colWidth= 0;
	int numTopBlanks = 0;
	boolean main = false;	
	
    MouseHandler mh= new MouseHandler();
    JPopupMenu popEntry;
    ValueChart chart;
    
    JLabel selected_entry;

	public EntryNamePanel (Vector entryList, int colWidth, int numTopBlanks, boolean main, ValueChart chart){
		labelList = new Vector();
	
		int maxheight = 0;
		addMouseListener(mh);
		addMouseMotionListener(mh);
 
		setLayout (new BoxLayout (this, BoxLayout.X_AXIS));
		add(Box.createHorizontalGlue());

		for (Iterator it=entryList.iterator(); it.hasNext(); ){
			ChartEntry entry = (ChartEntry)it.next();
			labelList.add (entry.name);
		}
		
		setBorder(BorderFactory.createLineBorder(getBackground()));

  	   	panelHeight = maxheight;
		this.setPreferredSize(new Dimension(labelList.size()*colWidth+ ANG_WD, ANG_HT + 3));
		this.setMaximumSize(new Dimension(labelList.size()*colWidth + ANG_WD, ANG_HT + 3));
		this.setMinimumSize(new Dimension(labelList.size()*colWidth + ANG_WD, ANG_HT + 3));
  	   	this.colWidth = colWidth;
  	   	//setAlignmentY (JComponent.TOP_ALIGNMENT);
  	   	this.numTopBlanks = numTopBlanks;
  	   	this.main = main;	
  	   	this.chart = chart;
	   	
  	   	createPopup();
  	   	
	   }

	int mousedOver = -1;
	
    public void paintComponent(Graphics g) {
    	((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
    	        RenderingHints.VALUE_ANTIALIAS_ON);
    	g.setColor(getBackground());
        g.fillRect(0, 0, getWidth(), getHeight());	
        g.setColor(Color.BLACK);
        g.setFont(VLABELFONT);
        for(int i = 0; i < labelList.size(); ++i) {    
        	if (i == mousedOver)
        		g.setColor(Color.GRAY);
        	else
        		g.setColor(Color.BLACK);
        	g.drawString(labelList.get(i).toString(), (i*colWidth) + 20, ANG_HT);
        	g.setColor(Color.GRAY);
        	g.drawLine((i*colWidth), ANG_HT, (i*colWidth) + ANG_WD, 0);
        	if (((ChartEntry)chart.entryList.get(i)).getIsMarked()){
        		g.setColor(Color.blue);
        		//code a polygon or something
        		g.drawString(labelList.get(i).toString(), (i*colWidth) + 21, ANG_HT - 1);        		
        	}
        }
        g.drawLine((labelList.size()*colWidth), ANG_HT, (labelList.size()*colWidth) + ANG_WD, 0);
        g.drawLine(ANG_WD, 0, (labelList.size()*colWidth) + ANG_WD, 0);
        g.drawLine(0, ANG_HT, (labelList.size()*colWidth), ANG_HT);
    }  
	
    void createPopup(){     	
		popEntry = new JPopupMenu();		
	    JMenuItem menuItem = new JMenuItem("Mark");
	    menuItem.addActionListener(this);
	    popEntry.add(menuItem);
	    menuItem = new JMenuItem("Edit");
	    menuItem.addActionListener(this);	    
	    popEntry.add(menuItem);
	    popEntry.addSeparator();
	    menuItem = new JMenuItem("Show Details");
	    menuItem.addActionListener(this);
	    popEntry.add(menuItem);	    
    }
	
	public void relabel (Vector entryList){
		labelList.clear();
		
		for (Iterator ei=entryList.iterator(); ei.hasNext(); ){
			ChartEntry entry = (ChartEntry)ei.next();
			labelList.add(entry.name);
	    }

		repaint();
	 }

	public int getDesiredHeight (){
	   return panelHeight;
	}
	
	void updateBoth(){
		chart.mainEntryNames.repaint();
		chart.displayEntryNames.repaint();
	}
	
	public void actionPerformed(ActionEvent ae) {
		//listener for popup menu
		if ("Mark".equals(ae.getActionCommand()) || "Unmark".equals(ae.getActionCommand())){
			if("Mark".equals(ae.getActionCommand()))
				((ChartEntry)chart.entryList.get(rightClicked)).setIsMarked(true);
			else
				((ChartEntry)chart.entryList.get(rightClicked)).setIsMarked(false);
			updateBoth();
			/*			if (chart.mainEntryNames.isAncestorOf(selected_entry))
				markMatchingLabel(chart.displayEntryNames, col);
	        else			
	        	markMatchingLabel(chart.mainEntryNames, col);		*/	
		}
		else if ("Show Details".equals(ae.getActionCommand()) || "Hide Details".equals(ae.getActionCommand())){			
			if ("Show Details".equals(ae.getActionCommand()))
				toggleDetails(true);
			else
				toggleDetails(false);
		}
		
		else if ("Edit".equals(ae.getActionCommand()) || "Hide Details".equals(ae.getActionCommand())){			
			chart.showEditView(1);
		}	
	}
		Vector marked = new Vector();
		/**/
		void markMatchingLabel(EntryNamePanel enp, Color color){
			for (int i=0; i<enp.getComponentCount(); i++){
				JLabel labi = (JLabel)enp.getComponent(i);
				if (labi.getText().equals(selected_entry.getText())){
						JLabel lab = (JLabel)enp.getComponent(i);
						lab.setBackground(color);
						break;
				}
				
			}
		}
		
		void toggleDetails(boolean show){
			//offset fom extra label added for ruler
			((ChartEntry)chart.entryList.get(rightClicked)).setShowFlag(show);	
			chart.updateAll();	
		}
			
		/**/
	    public Dimension getPreferredSize() {
	    	return new Dimension(labelList.size()*colWidth + ANG_WD, ANG_HT + 3);
	    }

	    int rightClicked = -1;
	class MouseHandler extends MouseInputAdapter{
		int idx=0;
		public void mousePressed(MouseEvent me){
			//((ChartEntry)chart.entryList.get(idx)).setShowFlag(true);	
			if (idx != -1)
				chart.showDomainVals(idx);
			//System.out.println("IDX___________ " + idx);
			chart.updateAll();	
			if(SwingUtilities.isRightMouseButton(me)){
					if (((ChartEntry)chart.entryList.get(idx)).getIsMarked()){
						((JMenuItem)popEntry.getComponent(0)).setText("Unmark");
					}
					else
						((JMenuItem)popEntry.getComponent(0)).setText("Mark");						
				if(idx!=-1){
					if(((ChartEntry)chart.entryList.get(idx)).getShowFlag())
							((JMenuItem)popEntry.getComponent(3)).setText("Hide Details");
						else
							((JMenuItem)popEntry.getComponent(3)).setText("Show Details");
				}				
     			rightClicked = idx;
				popEntry.show(me.getComponent(), me.getX()-1, me.getY()-1);            	
     			
            }
		}		
		
		public void mouseMoved(MouseEvent me){
			int inc = ANG_WD / 4;
			int hity = 0;
			int y=getHeight()/4;
			for (int i=0; i<4; i++){
				if(me.getY()>(i*y) && me.getY()<(i+1)*y){
					hity = (3-i) * inc;
					break;
				}
			}
			for (int i=0; i<labelList.size(); i++){
				if(me.getX() > ((i*colWidth) + hity) && me.getX() <
						((i*colWidth) + hity + colWidth)){
					idx = i;
					EntryNamePanel.this.setToolTipText(labelList.get(i).toString());
					break;
				}	
				//no sel
				idx = -1;
			}
			mousedOver=idx;
			
			if (me.getY() < 3 || me.getY() > getHeight()-3)
				mousedOver = -1;
			
			repaint();
		}
	
		public void mouseExited(MouseEvent me){
			mousedOver = -1;
			repaint();
		}

	}
	

	
}
